@@ -6,9 +6,10 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file dialog_initial.c
+	@brief Main code for the application window.
 	@author Copyright 2006-2007  Neil Williams <linux@codehelp.co.uk>
 	@author Copyright 1999  Robert Lissner, Jay MacDonald,
-	@author Copyright 1999  Sam Phillips, Keith Wesolowski 
+	@author Copyright 1999  Sam Phillips, Keith Wesolowski
  */
 /*
     This package is free software; you can redistribute it and/or modify
@@ -30,6 +31,7 @@
 #include <gtk/gtk.h>
 #include <gtkextra/gtksheet.h>
 #include "types.h"
+#include "field.h"
 #include "dialog_initial.h"
 #include "menus.h"
 #include "dim_list_menu.h"
@@ -96,7 +98,7 @@ main_window (QlContext * qlc)
 	gtk_notebook_set_scrollable (qlc->notebook, TRUE);
 	gtk_notebook_popup_enable (qlc->notebook);
 	gtk_notebook_set_tab_pos (qlc->notebook, GTK_POS_TOP);
-	gtk_box_pack_start (GTK_BOX(list_vbox), 
+	gtk_box_pack_start (GTK_BOX(list_vbox),
 		GTK_WIDGET (qlc->notebook), TRUE, TRUE, 0);
 	qlstatusbar = gtk_statusbar_new ();
 	gtk_box_pack_end (GTK_BOX (list_vbox), qlstatusbar, FALSE, FALSE, 0);
@@ -105,7 +107,7 @@ main_window (QlContext * qlc)
 }
 
 /** initialise a table of notebook tab data */
-QlContext * 
+QlContext *
 ql_context_init (void)
 {
 	QlContext * qlc;
@@ -140,6 +142,24 @@ ql_get_context (GtkWidget * widget)
 	return NULL;
 }
 
+static gboolean
+ql_filedata_new (QlTabData * tab)
+{
+	g_return_val_if_fail (tab, FALSE);
+	tab->file = g_new0 (QlFileData, 1);
+	tab->file->fields = g_hash_table_new (g_direct_hash, g_direct_equal);
+	return TRUE;
+}
+
+static gboolean
+ql_viewdata_new (QlTabData * tab)
+{
+	g_return_val_if_fail (tab, FALSE);
+	tab->view = g_new0 (QlViewData, 1);
+
+	return TRUE;
+}
+
 /** \brief initialise data for a notebook tab
 
 Associates a notebook tab with it's view (sheet or report)
@@ -153,11 +173,10 @@ ql_new_tabdata (QlContext * qlc)
 
 	nindex = 0;
 	tab = g_new0 (QlTabData, 1);
-	tab->view = g_new0 (QlViewData, 1);
-	tab->file = g_new0 (QlFileData, 1);
+	g_return_val_if_fail (ql_viewdata_new (tab), NULL);
+	g_return_val_if_fail (ql_filedata_new (tab), NULL);
 	tab->qlc = qlc;
-	nindex = gtk_notebook_get_current_page (qlc->notebook);
-	nindex++;
+	nindex = gtk_notebook_get_n_pages (qlc->notebook);
 	g_hash_table_insert (qlc->tab_table, GINT_TO_POINTER(nindex), tab);
 	return tab;
 }
@@ -178,7 +197,7 @@ ql_get_filedata (QlContext * qlc)
 	index = gtk_notebook_get_current_page (qlc->notebook);
 	if (index < 0)
 		return NULL;
-	tab = (QlTabData*)g_hash_table_lookup (qlc->tab_table, 
+	tab = (QlTabData*)g_hash_table_lookup (qlc->tab_table,
 		GINT_TO_POINTER(index));
 	return tab->file;
 }
@@ -194,7 +213,7 @@ ql_get_viewdata (QlContext * qlc)
 	index = gtk_notebook_get_current_page (qlc->notebook);
 	if (index < 0)
 		return NULL;
-	tab = (QlTabData*)g_hash_table_lookup (qlc->tab_table, 
+	tab = (QlTabData*)g_hash_table_lookup (qlc->tab_table,
 		GINT_TO_POINTER(index));
 	return tab->view;
 }
@@ -210,11 +229,67 @@ ql_get_tabdata (QlContext * qlc)
 	index = gtk_notebook_get_current_page (qlc->notebook);
 	if (index < 0)
 		return NULL;
-	tab = (QlTabData*)g_hash_table_lookup (qlc->tab_table, 
+	tab = (QlTabData*)g_hash_table_lookup (qlc->tab_table,
 		GINT_TO_POINTER(index));
 	return tab;
 }
 
+QlFieldInfo *
+ql_get_fieldinfo (QlTabData * tab, gint field_index)
+{
+	QlFieldInfo * field;
+
+	g_return_val_if_fail (tab, NULL);
+	field = (QlFieldInfo*)g_hash_table_lookup (tab->file->fields,
+		GINT_TO_POINTER(field_index));
+	return field;
+}
+
+gboolean
+ql_add_fieldinfo (QlTabData * tab, QlFieldInfo * field)
+{
+	gint size;
+	g_return_val_if_fail (tab, FALSE);
+	g_return_val_if_fail (field, FALSE);
+	size = g_hash_table_size (tab->file->fields);
+	g_hash_table_insert (tab->file->fields, GINT_TO_POINTER(size) ,field);
+	return TRUE;
+}
+
+gboolean
+ql_set_fieldinfo (QlTabData * tab, QlFieldInfo * field, gint field_index)
+{
+	g_return_val_if_fail (tab, FALSE);
+	g_return_val_if_fail (field, FALSE);
+	g_return_val_if_fail (g_hash_table_lookup(tab->file->fields,
+		GINT_TO_POINTER(field_index)), FALSE);
+	g_hash_table_replace (tab->file->fields, GINT_TO_POINTER (field_index), field);
+	return TRUE;
+}
+
+void
+ql_remove_fieldinfo (QlTabData * tab, gint field_index)
+{
+	g_return_if_fail (tab);
+	g_return_if_fail (g_hash_table_lookup(tab->file->fields,
+		GINT_TO_POINTER(field_index)));
+	g_hash_table_remove (tab->file->fields, GINT_TO_POINTER (field_index));
+}
+
+gint
+ql_get_last_field (QlTabData * tab)
+{
+	g_return_val_if_fail (tab, -1);
+	return g_hash_table_size (tab->file->fields);
+}
+
+void
+ql_fieldinfo_foreach (QlTabData * tab, QlFieldInfoCB cb, gpointer user_data)
+{
+	g_return_if_fail (tab);
+	g_hash_table_foreach (tab->file->fields, cb, user_data);
+}
+
 void
 ql_free_context (QlContext *qlc)
 {
