@@ -285,8 +285,10 @@ dim_list_report_menu (QlContext * qlc)
 	tab = ql_get_tabdata (qlc);
 	if (tab)
 	{
-		if (tab->file->report_ct)
+		if (tab->file->report_ct > 0)
+		{
 			mselect = TRUE;
+		}
 	}
 	menus_set_sensitive (qlc, "/menubar/FileMenu/Print", TRUE);
 	menus_set_sensitive (qlc, "/menubar/FileMenu/Preview", TRUE);
@@ -357,29 +359,29 @@ void
 dim_report_column_menu (QlContext * qlc)
 {
 	QlTabData * tab;
-	gboolean add = FALSE;
-	gboolean delete = FALSE;
-	gboolean right = FALSE;
-	gboolean left = FALSE;
+	gboolean rep_add = FALSE;
+	gboolean rep_delete = FALSE;
+	gboolean rep_right = FALSE;
+	gboolean rep_left = FALSE;
 
 	tab = ql_get_tabdata (qlc);
 	if (tab->file->report_ptr->last_column < MAX_FIELDS - 1)
-		add = TRUE;
+		rep_add = TRUE;
 
 	if (tab->view->report_sel_col >= 0)
 		if (tab->file->report_ptr->last_column)
 		{
 			/* ie two or more columns */
-			delete = TRUE;
+			rep_delete = TRUE;
 			if (tab->view->report_sel_col < tab->file->report_ptr->last_column)
-				right = TRUE;
+				rep_right = TRUE;
 			if (tab->view->report_sel_col)
-				left = TRUE;
+				rep_left = TRUE;
 		}
-	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnAdd", add);
-	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnDelete", delete);
-	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnRight", right);
-	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnLeft", left);
+	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnAdd", rep_add);
+	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnDelete", rep_delete);
+	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnRight", rep_right);
+	menus_set_sensitive (qlc, "/menubar/ColumnMenu/ColumnLeft", rep_left);
 }	/* end of dim_report_column_menu */
 
 void
@@ -392,8 +394,10 @@ dim_report_totals_menu (QlContext * qlc)
 	tab = ql_get_tabdata (qlc);
 	if (tab->view->report_sel_col >= 0)
 	{
+		QlFieldInfo * field;
 		group = TRUE;
-		if (tab->file->fields[tab->view->report_sel_field].type == 2)
+		field = ql_get_fieldinfo (tab, tab->view->report_sel_field);
+		if (field->type == 2)
 			total = TRUE;
 	}
 /*
