@@ -6,6 +6,7 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file edit.c
+	@brief Functions for edit - cut, copy and paste.
 	@author Copyright 2006-2007  Neil Williams <linux@codehelp.co.uk>
 	@author Copyright 1999  Robert Lissner
 */
@@ -338,13 +339,13 @@ void
 new_column_width (GtkSheet G_GNUC_UNUSED * sheet, gint col, 
 				  gint width, gpointer data)
 {
-	gint fieldx;
 	QlTabData * tab;
+	QlFieldInfo * field;
 
 	tab = (QlTabData*)data;
 	front_is_changed (tab);
-	fieldx = tab->file->col_to_field[col];
-	tab->file->fields[fieldx].width = width / 8;
+	field = ql_get_fieldinfo (tab, tab->file->col_to_field[col]);
+	field->width = width / 8;
 }
 
 /** \todo does QL really need to manage the clipboard itself? */
