@@ -6,9 +6,10 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file field.c
+	@brief  Add and edit fields (columns) in the GtkSheet.
 	@author Copyright 2006-2007  Neil Williams <linux@codehelp.co.uk>
 	@author Copyright 1999  Robert Lissner, Jay MacDonald
-	@author Copyright 1999  Sam Phillips, Keith Wesolowski. 
+	@author Copyright 1999  Sam Phillips, Keith Wesolowski.
 */
 /*
     This package is free software; you can redistribute it and/or modify
@@ -33,6 +34,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gtkextra/gtksheet.h>
+#include <monetary.h>
 #include "types.h"
 #include "dialog_initial.h"
 #include "dim_list_menu.h"
@@ -40,8 +42,8 @@
 #include "main.h"
 
 /* this is all done in a dialog box, so globals are OK */
-static QlFieldInfo field_db_start;
-static QlFieldInfo field_db_curr;
+static QlFieldInfo * field_db_start;
+static QlFieldInfo * field_db_curr;
 static GtkWidget *done_button;
 /** so we can ignore signals during display */
 static gboolean displaying;
@@ -118,14 +120,14 @@ done_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 static void
 just_clicked (GtkObject G_GNUC_UNUSED * object, gpointer entry)
 {
-	field_db_curr.justification = GPOINTER_TO_INT (entry);
+	field_db_curr->justification = GPOINTER_TO_INT (entry);
 }
 
 static void
-sm_formatting_changed (GtkObject G_GNUC_UNUSED * object, 
+sm_formatting_changed (GtkObject G_GNUC_UNUSED * object,
 					   gpointer G_GNUC_UNUSED entry)
 {
-	field_db_curr.formatting =
+	field_db_curr->formatting =
 		gtk_combo_box_get_active (GTK_COMBO_BOX (object));
 }
 
@@ -140,35 +142,35 @@ update_field_db (QlTabData * tab)
 
 	text = gtk_entry_get_text (GTK_ENTRY (column_name_entry));
 	if (text && strlen (text) > 0)
-		strcpy (field_db_curr.name, text);
-	gtk_entry_set_text (GTK_ENTRY (column_name_entry), field_db_curr.name);
+		field_db_curr->name = g_strdup(text);
+	gtk_entry_set_text (GTK_ENTRY (column_name_entry), field_db_curr->name);
 	/* otherwise assume want to enter name */
 	if (tab->view->dialog_mode != MODE_EDIT)
 		gtk_editable_select_region (GTK_EDITABLE (column_name_entry), 0,
-			strlen (field_db_curr.name));
+			strlen (field_db_curr->name));
 	gtk_widget_hide (GTK_WIDGET (tab->view->sm_numeric_box));
 	gtk_widget_hide (tab->view->dec_places_label);
 	gtk_widget_hide (decimal_places_entry);
 	gtk_widget_hide (GTK_WIDGET (tab->view->sm_date_box));
 	gtk_widget_hide (GTK_WIDGET (tab->view->sm_time_box));
 
-	switch (field_db_curr.type)
+	switch (field_db_curr->type)
 	{
 		case FIELD_TYPE_TEXT:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (text_button), TRUE);
 			break;
 		}
 		case FIELD_TYPE_NUMERIC:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 			(GTK_TOGGLE_BUTTON (numeric_button), TRUE);
 			gtk_combo_box_set_active (tab->view->sm_numeric_box,
-				field_db_curr.formatting);
-			if (field_db_curr.decimal_places >= 0 &&
-				field_db_curr.decimal_places < 10)
-				sprintf (linebuf, "%u", field_db_curr.decimal_places);
+				field_db_curr->formatting);
+			if (field_db_curr->decimal_places >= 0 &&
+				field_db_curr->decimal_places < 10)
+				sprintf (linebuf, "%u", field_db_curr->decimal_places);
 			else
 				strcpy (linebuf, "0");
 
@@ -180,40 +182,40 @@ update_field_db (QlTabData * tab)
 		}
 		case FIELD_TYPE_DATE:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (date_button), TRUE);
 			gtk_combo_box_set_active (tab->view->sm_date_box,
-				field_db_curr.formatting);
+				field_db_curr->formatting);
 			gtk_widget_show (GTK_WIDGET (tab->view->sm_date_box));
 			break;
 		}
 		case FIELD_TYPE_TIME:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (time_button), TRUE);
 			gtk_combo_box_set_active (tab->view->sm_time_box,
-				field_db_curr.formatting);
+				field_db_curr->formatting);
 			gtk_widget_show (GTK_WIDGET (tab->view->sm_time_box));
 			break;
 		}
 	}
-	switch (field_db_curr.justification)
+	switch (field_db_curr->justification)
 	{
 		case GTK_JUSTIFY_LEFT:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (left_button), TRUE);
 			break;
 		}
 		case GTK_JUSTIFY_CENTER:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (center_button), TRUE);
 			break;
 		}
 		case GTK_JUSTIFY_RIGHT:
 		{
-			gtk_toggle_button_set_active 
+			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (right_button), TRUE);
 			break;
 		}
@@ -235,12 +237,12 @@ type_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 
 	tab = (QlTabData*) g_object_get_data (G_OBJECT(entry_dialogue), QLTABID);
 	g_return_if_fail (tab);
-	field_db_curr.type = GPOINTER_TO_INT (data);
-	field_db_curr.formatting = 0;
-	if (field_db_curr.type == FIELD_TYPE_TEXT)
-		field_db_curr.justification = GTK_JUSTIFY_LEFT;
+	field_db_curr->type = GPOINTER_TO_INT (data);
+	field_db_curr->formatting = 0;
+	if (field_db_curr->type == FIELD_TYPE_TEXT)
+		field_db_curr->justification = GTK_JUSTIFY_LEFT;
 	else
-		field_db_curr.justification = GTK_JUSTIFY_RIGHT;
+		field_db_curr->justification = GTK_JUSTIFY_RIGHT;
 	update_field_db (tab);
 }
 
@@ -274,7 +276,45 @@ get_timestamp (const gchar * format)
 	return d;
 }
 
-/** Assemble the style menus for numeric, date and time */
+/** \brief used for currency
+
+ \param number The amount of currency.
+ \param qc The QuickList currency format to use.
+*/
+static gchar *
+display_currency (gdouble number, QlCurrFormat qc)
+{
+	gchar buf[MAX_DATE_BUFFER];
+	gsize len;
+	gchar * format;
+
+	format = NULL;
+	switch (qc)
+	{
+		/* "1,234.56" %! formatting style for formatting == 0 */
+		case QL_NF_GRP :   { format = g_strdup("%!i"); break;  }
+		/* "1234.56"  %^! */
+		case QL_NF :       { format = g_strdup("%^!i"); break; }
+		/* "$1,234.56"  default (%n) */
+		case QL_NF_CURR :  { format = g_strdup("%n"); break;  }
+		/* "1,234.56%" percentage numbers */
+		case QL_NF_PERCENT : { format = g_strdup ("%!i%%"); break; }
+		/* "1.234,56" used for files only */
+		/* "1234,56" used for files only */
+		/* "1.234,56%" used for files only */
+		default : break;
+	}
+	if (!format)
+		return NULL;
+	buf[0] = '\1';
+	len = strfmon (buf, MAX_DATE_BUFFER, format, number);
+	g_free (format);
+	if (len <= 0 && buf[0] != '\0')
+		return NULL;
+	return g_strdup (buf);
+}
+
+/** \brief Assemble the style menus for numeric, date and time */
 static GtkWidget *
 sub_menus (QlTabData * tab)
 {
@@ -282,7 +322,7 @@ sub_menus (QlTabData * tab)
 	gint subx;
 
 	/* add the individual display types to the menu.  We'll show them
-	   later.  They only show one at a time. */
+	 later.  They only show one at a time. */
 	sub_menu_box = gtk_hbox_new (FALSE, 5);
 	gtk_widget_show (sub_menu_box);
 
@@ -291,19 +331,18 @@ sub_menus (QlTabData * tab)
 	tab->view->sm_numeric_box = GTK_COMBO_BOX (gtk_combo_box_new_text ());
 	for (subx = 0; subx < QL_NF_LAST; subx++)
 	{
-		gchar * nstr = display_number_str(subx, 1234.56);
+		gchar * nstr = display_currency(1234.56, subx);
 		if (!nstr)
 			continue;
-		gtk_combo_box_insert_text (tab->view->sm_numeric_box, subx,	nstr);
+		gtk_combo_box_insert_text (tab->view->sm_numeric_box, subx, nstr);
 	}
 	g_signal_connect (GTK_OBJECT (tab->view->sm_numeric_box), "changed",
 		G_CALLBACK (sm_formatting_changed), &subx);
 	gtk_box_pack_start (GTK_BOX (sub_menu_box),
 		GTK_WIDGET (tab->view->sm_numeric_box), FALSE, FALSE, 0);
 
-	decimal_places_entry = gtk_entry_new ();
-	gtk_entry_set_max_length (GTK_ENTRY (decimal_places_entry), 1);
-	gtk_widget_set_size_request (decimal_places_entry, 40, -1);
+	decimal_places_entry = gtk_spin_button_new_with_range (0,9,1);
+	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(decimal_places_entry), TRUE);
 	gtk_box_pack_end (GTK_BOX (sub_menu_box), decimal_places_entry, FALSE, FALSE,
 		0);
 	tab->view->dec_places_label = gtk_label_new (_("Decimal places"));
@@ -346,17 +385,32 @@ new_std_field (QlTabData * tab)
 {
 	gtk_entry_set_text (GTK_ENTRY (column_name_entry), "");
 	/* if last_field =2, then new is Field 4 */
-	g_snprintf (field_db_curr.name, MAX_FIELD_NAME,
-		_("Column %u"), tab->file->last_field + 2);
-	field_db_curr.sheet_column = tab->file->last_field + 1;
-	field_db_curr.type = FIELD_TYPE_TEXT;
-	field_db_curr.formatting = 0;
-	field_db_curr.decimal_places = 0;
-	field_db_curr.justification = GTK_JUSTIFY_LEFT;
-	field_db_curr.width = 10;
+	field_db_curr->name = g_strdup_printf (_("Column %u"),
+		tab->file->last_field + 2);
+	field_db_curr->sheet_column = tab->file->last_field + 1;
+	field_db_curr->type = FIELD_TYPE_TEXT;
+	field_db_curr->formatting = 0;
+	field_db_curr->decimal_places = 0;
+	field_db_curr->justification = GTK_JUSTIFY_LEFT;
+	field_db_curr->width = 10;
 }
 
-/** Remember that sheet isn't open yet if this is new file */
+static void
+add_column_cb (gpointer G_GNUC_UNUSED field_index, gpointer field_ptr, gpointer user_data)
+{
+	gint sheet_column;
+	QlFieldInfo * field = (QlFieldInfo*)field_ptr;
+	sheet_column = GPOINTER_TO_INT (user_data);
+	if (field->sheet_column >= sheet_column)
+		field->sheet_column++;
+}
+
+/** \brief Create new field in the sheet
+
+Called by ::build_field_db and therefore used by new and
+existing files. Remember that the sheet isn't open yet if
+this is a new file.
+*/
 static void
 ok_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 {
@@ -377,42 +431,32 @@ ok_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 	if (check_entry (tab, tmp))
 		return;
 	g_free (tmp);
-	strcpy (field_db_curr.name, text);
+	field_db_curr->name = g_strdup(text);
 
-	if (field_db_curr.type == FIELD_TYPE_NUMERIC)
+	if (field_db_curr->type == FIELD_TYPE_NUMERIC)
 	{
-		text = gtk_entry_get_text (GTK_ENTRY (decimal_places_entry));
-		if (text[0] < '0' || text[0] > '9' || text[1])
-		{
-			level2_error (tab, _("The number of decimal places must be "
-					"between 0 to 9"));
-			return;
-		}
-		field_db_curr.decimal_places = atoi (text);
+		field_db_curr->decimal_places = gtk_spin_button_get_value_as_int
+			(GTK_SPIN_BUTTON(decimal_places_entry));
 	}
 
-	/* notation is just too complicated */
-	/* A is add one column */
+	/* we have an existing file and we are adding a new field/column */
 	if (tab->view->dialog_mode == MODE_ADD)
 	{
-		field_db_curr.sheet_column = sheetx = tab->view->sel_range.col0;
-
-		for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
-			if (tab->file->fields[fieldx].sheet_column >= sheetx)
-				tab->file->fields[fieldx].sheet_column++;
+		field_db_curr->sheet_column = sheetx = tab->view->sel_range.col0;
+		ql_fieldinfo_foreach (tab, add_column_cb, GINT_TO_POINTER (sheetx));
 
 		fieldx = ++tab->file->last_field;
 		big_draw_start (tab);
-		tab->file->fields[fieldx] = field_db_curr;
+		ql_add_fieldinfo (tab, field_db_curr);
 
 		/* add column, add to sheet now */
 		gtk_sheet_insert_columns (tab->view->sheet, sheetx, 1);
 		gtk_sheet_set_column_width (tab->view->sheet, sheetx,
-			field_db_curr.width * 8);
+			field_db_curr->width * 8);
 		gtk_sheet_column_button_add_label (tab->view->sheet,
-			sheetx, field_db_curr.name);
+			sheetx, field_db_curr->name);
 		gtk_sheet_column_set_justification (tab->view->sheet,
-			sheetx, field_db_curr.justification);
+			sheetx, field_db_curr->justification);
 
 		reset_col_to_field (tab);
 		big_draw_end (tab);
@@ -420,55 +464,57 @@ ok_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 		return;
 	}							/* end of add field */
 
-	/* now do edit field */
+	/* we have an existing file and editing the fields. */
 	else if (tab->view->dialog_mode == MODE_EDIT)
 	{
+		QlFieldInfo * field;
 		sheetx = tab->view->sel_range.col0;
-		fieldx = tab->file->col_to_field[sheetx];
-		tab->file->fields[fieldx] = field_db_curr;
+		field = ql_get_fieldinfo (tab, sheetx);
+		field = field_db_curr;
 		gtk_sheet_column_button_add_label (tab->view->sheet,
-			sheetx, field_db_curr.name);
+			sheetx, field_db_curr->name);
 		big_draw_start (tab);
-		if (field_db_curr.justification != field_db_start.justification)
+		if (field_db_curr->justification != field_db_start->justification)
 		{
 			gtk_sheet_column_set_justification (tab->view->sheet,
-				sheetx, field_db_curr.justification);
+				sheetx, field_db_curr->justification);
 			range.row0 = 0;
 			range.rowi = tab->file->last_row;
 			range.col0 = range.coli = sheetx;
 			gtk_sheet_range_set_justification (tab->view->sheet,
-				&range, field_db_curr.justification);
+				&range, field_db_curr->justification);
 		}
 
 		/* check if numeric format changed */
-		if (field_db_curr.type == FIELD_TYPE_TEXT || 
+		if (field_db_curr->type == FIELD_TYPE_TEXT ||
 			!tab->file->last_row)
 		{
 			big_draw_end (tab);
 			front_is_changed (tab);
 			return;
 		}
-		if (field_db_curr.type != FIELD_TYPE_TEXT &&
-			(field_db_start.type != field_db_curr.type ||
-				field_db_start.formatting != field_db_curr.formatting ||
-				field_db_start.decimal_places !=
-				field_db_curr.decimal_places))
+		if (field_db_curr->type != FIELD_TYPE_TEXT &&
+			(field_db_start->type != field_db_curr->type ||
+				field_db_start->formatting != field_db_curr->formatting ||
+				field_db_start->decimal_places !=
+				field_db_curr->decimal_places))
 		{
 
 			/* make sure the old formatting code is valid */
-			if (field_db_start.type == FIELD_TYPE_TEXT)
-				field_db_start.formatting = field_db_curr.formatting;	/* default */
+			/** \bug need type-specific GList based formatting. */
+			if (field_db_start->type == FIELD_TYPE_TEXT)
+				field_db_start->formatting = field_db_curr->formatting;	/* default */
 			for (rowx = 0; rowx < tab->file->last_row; rowx++)
 			{
 				text = gtk_sheet_cell_get_text (tab->view->sheet,
 					rowx, sheetx);
 				if (text)
 				{
-					temp_double = qls2d (text, field_db_curr.type,
-						field_db_start.formatting);
-					d2qls (linebuf, temp_double, field_db_curr.type,
-						field_db_curr.formatting,
-						field_db_curr.decimal_places);
+					temp_double = qls2d (text, field_db_curr->type,
+						field_db_start->formatting);
+					d2qls (linebuf, temp_double, field_db_curr->type,
+						field_db_curr->formatting,
+						field_db_curr->decimal_places);
 					gtk_sheet_set_cell_text (tab->view->sheet,
 						rowx, sheetx, linebuf);
 				}
@@ -479,9 +525,9 @@ ok_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 		return;
 	}
 
-	/* this is for a new file */
+	/* we have a new file */
 	fieldx = sheetx = ++tab->file->last_field;
-	tab->file->fields[fieldx] = field_db_curr;
+	ql_add_fieldinfo (tab, field_db_curr);
 	reset_col_to_field (tab);
 	new_std_field (tab);
 	field_db_start = field_db_curr;
@@ -491,38 +537,33 @@ ok_clicked (GtkObject G_GNUC_UNUSED * object, gpointer data)
 static void
 swap_columns (QlTabData * tab, gint tocol, gint from)
 {
+	QlFieldInfo * fieldto, * fieldfrom;
 	gchar *textfromp;
 	gchar *texttop;
 	gchar textfrom[1024];
-	gint fieldfrom;
-	gint fieldto;
 	gint from_justification;
 	gint to_justification;
 	gint rowx;
 	gint to_sheet_col;
 
 	big_draw_start (tab);
-	fieldfrom = tab->file->col_to_field[from];
-	fieldto = tab->file->col_to_field[tocol];
-	from_justification = tab->file->fields[fieldfrom].justification;
-	to_justification = tab->file->fields[fieldto].justification;
-	to_sheet_col = tab->file->fields[fieldto].sheet_column;
-	tab->file->fields[fieldto].sheet_column =
-		tab->file->fields[fieldfrom].sheet_column;
-	tab->file->fields[fieldfrom].sheet_column = to_sheet_col;
+	fieldfrom = ql_get_fieldinfo (tab, from);
+	fieldto = ql_get_fieldinfo (tab, tocol);
+	from_justification = fieldfrom->justification;
+	to_justification = fieldto->justification;
+	to_sheet_col = fieldto->sheet_column;
+	fieldto->sheet_column = fieldfrom->sheet_column;
+	fieldfrom->sheet_column = to_sheet_col;
 
-	gtk_sheet_set_column_width (tab->view->sheet, from,
-		tab->file->fields[fieldto].width * 8);
-	gtk_sheet_set_column_width (tab->view->sheet, tocol,
-		tab->file->fields[fieldfrom].width * 8);
-	gtk_sheet_column_button_add_label (tab->view->sheet, from,
-		tab->file->fields[fieldto].name);
+	gtk_sheet_set_column_width (tab->view->sheet, from, fieldto->width * 8);
+	gtk_sheet_set_column_width (tab->view->sheet, tocol, fieldfrom->width * 8);
+	gtk_sheet_column_button_add_label (tab->view->sheet, from, fieldto->name);
 	gtk_sheet_column_button_add_label (tab->view->sheet, tocol,
-		tab->file->fields[fieldfrom].name);
+		fieldfrom->name);
 	gtk_sheet_column_set_justification (tab->view->sheet, from,
-		tab->file->fields[fieldto].justification);
+		fieldto->justification);
 	gtk_sheet_column_set_justification (tab->view->sheet, tocol,
-		tab->file->fields[fieldfrom].justification);
+		fieldfrom->justification);
 
 	reset_col_to_field (tab);
 
@@ -555,7 +596,7 @@ swap_columns (QlTabData * tab, gint tocol, gint from)
 	gtk_sheet_select_column (tab->view->sheet, tocol);
 }
 
-/** \brief Builds dialogue box to edit field parameters 
+/** \brief Builds dialogue box to edit field parameters
 
 \param front->sel_range.col0 the field being edited
 \param field_db_curr  the information we want to display
@@ -610,7 +651,7 @@ build_field_db (QlTabData * tab)
 	gtk_widget_show (text_button);
 
 	numeric_button = gtk_radio_button_new_with_label
-		(gtk_radio_button_get_group 
+		(gtk_radio_button_get_group
 		(GTK_RADIO_BUTTON (text_button)), _("Numeric"));
 	g_signal_connect (GTK_OBJECT (numeric_button), "clicked",
 		G_CALLBACK (type_clicked), GINT_TO_POINTER (FIELD_TYPE_NUMERIC));
@@ -619,7 +660,7 @@ build_field_db (QlTabData * tab)
 	gtk_widget_show (numeric_button);
 
 	date_button = gtk_radio_button_new_with_label
-		(gtk_radio_button_get_group 
+		(gtk_radio_button_get_group
 		(GTK_RADIO_BUTTON (text_button)), _("Date"));
 	g_signal_connect (GTK_OBJECT (date_button), "clicked",
 		G_CALLBACK (type_clicked), GINT_TO_POINTER (FIELD_TYPE_DATE));
@@ -628,7 +669,7 @@ build_field_db (QlTabData * tab)
 	gtk_widget_show (date_button);
 
 	time_button = gtk_radio_button_new_with_label
-		(gtk_radio_button_get_group 
+		(gtk_radio_button_get_group
 		(GTK_RADIO_BUTTON (text_button)), _("Time"));
 	g_signal_connect (GTK_OBJECT (time_button), "clicked",
 		G_CALLBACK (type_clicked), GINT_TO_POINTER (FIELD_TYPE_TIME));
@@ -648,7 +689,7 @@ build_field_db (QlTabData * tab)
 	gtk_widget_show (left_button);
 
 	center_button = gtk_radio_button_new_with_label
-		(gtk_radio_button_get_group 
+		(gtk_radio_button_get_group
 		(GTK_RADIO_BUTTON (left_button)), _("Center"));
 	g_signal_connect (GTK_OBJECT (center_button), "clicked",
 		G_CALLBACK (just_clicked), GINT_TO_POINTER (GTK_JUSTIFY_CENTER));
@@ -657,7 +698,7 @@ build_field_db (QlTabData * tab)
 	gtk_widget_show (center_button);
 
 	right_button = gtk_radio_button_new_with_label
-		(gtk_radio_button_get_group 
+		(gtk_radio_button_get_group
 		(GTK_RADIO_BUTTON (left_button)), _("Right"));
 	g_signal_connect (GTK_OBJECT (right_button), "clicked",
 		G_CALLBACK (just_clicked), GINT_TO_POINTER (GTK_JUSTIFY_RIGHT));
@@ -729,15 +770,26 @@ column_edit (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	tab->view->dialog_mode = MODE_EDIT;
 	selected = tab->view->sel_range.col0;
 	this_field = tab->file->col_to_field[selected];
-	field_db_curr = tab->file->fields[this_field];
+	field_db_curr = ql_get_fieldinfo (tab, this_field);
 	build_field_db (tab);
 	update_field_db (tab);
 }
 
 void
-column_delete (GtkWidget G_GNUC_UNUSED * w, gpointer data)
+del_column_cb (gpointer field_index, gpointer field_ptr,
+	gpointer G_GNUC_UNUSED user_data)
 {
-	gint colx, fieldx, field_leaving, groupx;
+	QlFieldInfo * field = (QlFieldInfo*)field_ptr;
+	gint colx = GPOINTER_TO_INT (field_index);
+	g_return_if_fail (field);
+	if (field->sheet_column > colx)
+		field->sheet_column--;
+}
+
+void
+column_delete (GtkAction G_GNUC_UNUSED * w, gpointer data)
+{
+	gint colx, field_leaving, groupx;
 	gint linex, reportto, reportfrom, starting_reports;
 	gint colto, colfrom, starting_cols;
 	QlReportInfo temp_report;
@@ -746,7 +798,7 @@ column_delete (GtkWidget G_GNUC_UNUSED * w, gpointer data)
 
 	qlc = ql_get_context (GTK_WIDGET(data));
 	tab = ql_get_tabdata (qlc);
-	/* someday we should have code to ask if the user 
+	/* someday we should have code to ask if the user
 	really wants to delete a column. */
 	CHECK_RANGE;
 	CHECK_CHANGED(tab);
@@ -755,13 +807,15 @@ column_delete (GtkWidget G_GNUC_UNUSED * w, gpointer data)
 	field_leaving = tab->file->col_to_field[colx];
 	gtk_sheet_delete_columns (tab->view->sheet, colx, 1);
 
-	for (fieldx = field_leaving; fieldx < tab->file->last_field; fieldx++)
-		tab->file->fields[fieldx] = tab->file->fields[fieldx + 1];
+/*	for (fieldx = field_leaving; fieldx < tab->file->last_field; fieldx++)
+		tab->file->fields[fieldx] = tab->file->fields[fieldx + 1];*/
 
 	tab->file->last_field--;
-	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
-		if (tab->file->fields[fieldx].sheet_column > colx)
-			tab->file->fields[fieldx].sheet_column--;
+	ql_remove_fieldinfo (tab, field_leaving);
+	ql_fieldinfo_foreach (tab, del_column_cb, GINT_TO_POINTER(colx));
+/*	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
+		if (tab->file->fields[fieldx]->sheet_column > colx)
+			tab->file->fields[fieldx]->sheet_column--;*/
 
 	reset_col_to_field (tab);
 
@@ -863,6 +917,8 @@ new_file (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	/* Translators: this is the name of a new file before being saved with a name */
 	tab->file->file_path = g_strconcat ("~/", _("Untitled_New_List"), ".qlf", NULL);
 	tab->file->file_name = g_strconcat (_("Untitled_New_List"), ".qlf", NULL);
+	field_db_curr = g_new0(QlFieldInfo, 1);
+	field_db_start = g_new0(QlFieldInfo, 1);
 	build_field_db (tab);
 	new_std_field (tab);
 	update_field_db (tab);
