@@ -6,6 +6,7 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file filein.c
+	@brief  Read data from supported file formats.
 	@author Copyright 2006-2007  Neil Williams <linux@codehelp.co.uk>
 	@author Copyright 1999 Robert Lissner
  */
@@ -78,7 +79,7 @@
 
 /** \brief Inherited error codes
 
-Previous error codes have been expanded into 
+Previous error codes have been expanded into
 more descriptive enum values that are then mapped
 as strings.
 
@@ -124,33 +125,36 @@ AS_STRING_FUNC_NON_TYPEDEF (QlErrorVal, QL_ERRORS)
 static void
 define_last_field (QlTabData * tab)
 {
+	QlFieldInfo * field;
 	gint temp = tab->file->last_field;
-	g_snprintf (tab->file->fields[temp].name, MAX_FIELD_NAME, "Column %u",
-		temp + 1);
-	tab->file->fields[temp].sheet_column = temp;
-	tab->file->fields[temp].type = FIELD_TYPE_TEXT;
-	tab->file->fields[temp].formatting = 0;
-	tab->file->fields[temp].decimal_places = 0;
-	tab->file->fields[temp].justification = GTK_JUSTIFY_LEFT;
-	tab->file->fields[temp].width = 10;
+	field = ql_get_fieldinfo (tab, tab->file->last_field);
+	field->name = g_strdup_printf ("Column %u", temp + 1);
+	field->sheet_column = temp;
+	field->type = FIELD_TYPE_TEXT;
+	field->formatting = 0;
+	field->decimal_places = 0;
+	field->justification = GTK_JUSTIFY_LEFT;
+	field->width = 10;
 	tab->file->col_to_field[temp] = temp;
 }
 
-/** \brief Add 1 column to sheet.  
+/** \brief Add 1 column to sheet.
 
  This is necessary for reading unspecified input
   files such as comma, html and tab */
 static void
 add1field (QlTabData * tab)
 {
+	QlFieldInfo * field;
 	gint fieldx = ++tab->file->last_field;
 	define_last_field (tab);
 	gtk_sheet_add_column (tab->view->sheet, 1);
 	gtk_sheet_set_column_width (tab->view->sheet, fieldx, 80);
+	field = ql_get_fieldinfo (tab, fieldx);
 	gtk_sheet_column_button_add_label (tab->view->sheet, fieldx,
-		tab->file->fields[fieldx].name);
+		field->name);
 	gtk_sheet_set_column_title (tab->view->sheet,
-		fieldx, tab->file->fields[fieldx].name);
+		fieldx, field->name);
 }								/* end of add1field */
 
 
@@ -159,7 +163,7 @@ It is supposed to be a comma delimited file (CSV) */
 static enum QlErrorVal
 read_comma_file (QlTabData * tab)
 {
-	/** \bug remove need for linebuf 
+	/** \bug remove need for linebuf
 	using g_strsplit ?
 	*/
 	gchar linebuf[256];
@@ -202,7 +206,7 @@ read_comma_file (QlTabData * tab)
 					else
 						add1field (tab);
 				}
-				gtk_sheet_set_cell_text 
+				gtk_sheet_set_cell_text
 					(tab->view->sheet, rowx,
 					fieldx, linebuf);
 				linebufp = beginp;
@@ -272,8 +276,8 @@ read_ql_file (QlTabData * tab)
 					&tab->file->filter_ct,
 					&tab->file->report_ct,
 					&tab->view->width,
-					&tab->view->height, 
-					&tab->view->x, 
+					&tab->view->height,
+					&tab->view->x,
 					&tab->view->y) < 6)
 				return QL_BAD_QUICKLIST_DATA;
 			tab->file->last_field--;
@@ -287,41 +291,44 @@ read_ql_file (QlTabData * tab)
 		/* process FIELD input header record */
 		else if (g_ascii_strcasecmp (tokbuf, "FIELD") == 0)
 		{
+			QlFieldInfo * field;
 			guint16 field_type;
 			if (fieldx > tab->file->last_field)
 				return QL_NO_FIELD_TYPE;
+			field = g_new0(QlFieldInfo, 1);
 			if (sscanf (linebuf, "%*s%hu%hu%hu%hu%hu%hu%hu%hu%40[^\n\\]",
 					&field_type,
-					&tab->file->fields[fieldx].formatting,
-					&tab->file->fields[fieldx].decimal_places,
-					&tab->file->fields[fieldx].justification,
-					&tab->file->fields[fieldx].sheet_column,
-					&tab->file->fields[fieldx].width,
-					&tab->file->fields[fieldx].unused1,
-					&tab->file->fields[fieldx].unused2, tokbuf) != 9)
+					&field->formatting,
+					&field->decimal_places,
+					&field->justification,
+					&field->sheet_column,
+					&field->width,
+					&field->unused1,
+					&field->unused2, tokbuf) != 9)
 				return QL_UNABLE_TO_PARSE_FIELD_TYPE;
 			else
-				tab->file->fields[fieldx].type = field_type;
+				field->type = field_type;
+			ql_add_fieldinfo (tab, field);
 			tokbufp = tokbuf;
-			/** remove leading spaces 
-			
+			/** remove leading spaces
+
 			\bug replace with glib functions.
 			*/
 			while (*tokbufp == ' ')
 				tokbufp++;
-			if (tab->file->fields[fieldx].type < FIELD_TYPE_TEXT
-				|| tab->file->fields[fieldx].type > FIELD_TYPE_TIME
-				|| tab->file->fields[fieldx].formatting > 12
-				|| tab->file->fields[fieldx].decimal_places > 6
-				|| tab->file->fields[fieldx].justification > GTK_JUSTIFY_CENTER
-				|| tab->file->fields[fieldx].sheet_column > tab->file->last_field
-				|| tab->file->fields[fieldx].width > 80)
+			if (field->type < FIELD_TYPE_TEXT
+				|| field->type > FIELD_TYPE_TIME
+				|| field->formatting > 12
+				|| field->decimal_places > 6
+				|| field->justification > GTK_JUSTIFY_CENTER
+				|| field->sheet_column > tab->file->last_field
+				|| field->width > 80)
 				return QL_UNABLE_TO_PARSE_FIELD;
 
 			thislen = strlen (tokbufp);
 			if (thislen < 1 || thislen > MAX_FIELD_NAME)
 				return QL_FIELD_NAME_IS_TOO_LONG;
-			strcpy (tab->file->fields[fieldx].name, tokbufp);
+			field->name = g_strdup(tokbufp);
 			fieldx++;
 		}
 		/* convert the SORT record to memory */
@@ -354,7 +361,7 @@ read_ql_file (QlTabData * tab)
 			/* number of sort lines read */
 			tokbufp = tokbuf;
 			/** remove leading spaces
-			
+
 			\bug replace with glib functions.
 			*/
 			while (*tokbufp == ' ')
@@ -481,7 +488,7 @@ read_ql_file (QlTabData * tab)
 			thislen = strlen (tokbufp);
 			if (thislen < 1 || thislen > MAX_FIELD_NAME)
 				return QL_REPORT_FIELD_NAME_TOO_LONG;
-			strcpy (tab->file->reports[reportx].name, tokbufp);
+			tab->file->reports[reportx].name = g_strdup(tokbufp);
 			tab->file->reports[reportx].last_column = colx - 1;
 			reportx++;
 			colx = 0;			/* reset the report column indicator */
@@ -503,8 +510,9 @@ read_ql_file (QlTabData * tab)
 	/* fields are all OK so go build the basic window and sheet */
 	reset_col_to_field (tab);
 	build_basic_list_mode (tab);
+	tab->file->last_field = fieldx;
 
-	/* The file is already open, now read it in. 
+	/* The file is already open, now read it in.
 	   Make some effort to not load entirely blank records */
 	while (fgets (linebuf, sizeof (linebuf), f))
 	{
@@ -523,13 +531,19 @@ read_ql_file (QlTabData * tab)
 			singlechar = *linebufp;
 			if (linebufp != prevbufp)
 			{					/* ie, not a zero length field */
+				QlFieldInfo * field;
 				if (fieldx > tab->file->last_field)
 					return QL_TOO_MANY_FIELDS;
-
+				field = ql_get_fieldinfo (tab, fieldx);
+				if (!field)
+				{
+					fieldx++;
+					continue;
+				}
 				*linebufp = '\0';	/* to terminate string move */
 				got_data = TRUE;
 				gtk_sheet_set_cell_text (tab->view->sheet, rowx,
-					tab->file->fields[fieldx].sheet_column, prevbufp);
+					field->sheet_column, prevbufp);
 			}
 			prevbufp = ++linebufp;
 			fieldx++;
@@ -613,14 +627,14 @@ report_error (QlTabData * tab, enum QlErrorVal err_number)
 {
 	gchar * tmp;
 
-	tmp = g_strconcat (_("Error: "), QlErrorValasString(err_number), 
+	tmp = g_strconcat (_("Error: "), QlErrorValasString(err_number),
 		". ", _("This file is apparently not the type that you selected."),
 		NULL);
 	level1_error (tab, tmp);
 	g_free (tmp);
 }
 
-/** This callback is when someone clicks OK on a 
+/** This callback is when someone clicks OK on a
 file selection dialog box */
 void
 actually_open (const gchar * filename, QlDialogMode G_GNUC_UNUSED open_mode,
@@ -656,7 +670,7 @@ actually_open (const gchar * filename, QlDialogMode G_GNUC_UNUSED open_mode,
 	tab->view->dialog_mode = MODE_SAVE;
 }								/* end of actual open */
 
-/** retrieve path from the filename of 
+/** retrieve path from the filename of
 the current tab, if any. */
 static gchar *
 check_last_path (QlContext * qlc)
@@ -666,7 +680,7 @@ check_last_path (QlContext * qlc)
 	file = ql_get_filedata (qlc);
 	if (!file)
 		return NULL;
-	return g_path_get_dirname (file->file_path);	
+	return g_path_get_dirname (file->file_path);
 }
 
 /** allow the user to choose the file to open. */
@@ -692,7 +706,7 @@ any_open (QlDialogMode open_mode, QlFileType file_type, QlContext * qlc)
 			NULL, GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), 
+		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser),
 			filter);
 	}
 	else
@@ -707,7 +721,7 @@ any_open (QlDialogMode open_mode, QlFileType file_type, QlContext * qlc)
 	filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter, _("All files *"));
 	gtk_file_filter_add_pattern (filter, "*");
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), 
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser),
 		filter);
 	/* need tab index */
 	if (last_path)
@@ -721,6 +735,19 @@ any_open (QlDialogMode open_mode, QlFileType file_type, QlContext * qlc)
 		actually_open (filename, open_mode, file_type, qlc);
 }								/* end of any_open callback */
 
+static void
+set_sheet_column_cb (gpointer G_GNUC_UNUSED field_index, gpointer field_ptr, gpointer user_data)
+{
+	GtkSheet * sheet;
+	gint colx;
+	QlFieldInfo * field = (QlFieldInfo*)field_ptr;
+	sheet = (GtkSheet*) user_data;
+	colx = field->sheet_column;
+	gtk_sheet_set_column_width (sheet, colx, field->width * 8);
+	gtk_sheet_column_button_add_label (sheet, colx, field->name);
+	gtk_sheet_column_set_justification (sheet, colx, field->justification);
+}
+
 /** window initialiser */
 void
 build_basic_list_mode (QlTabData * tab)
@@ -747,16 +774,17 @@ build_basic_list_mode (QlTabData * tab)
 	gtk_sheet_show_row_titles (sheet);
 	gtk_sheet_show_column_titles (sheet);
 	gtk_sheet_rows_set_sensitivity (sheet, TRUE);
-	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
+	ql_fieldinfo_foreach (tab, set_sheet_column_cb, sheet);
+/*	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
 	{
-		colx = tab->file->fields[fieldx].sheet_column;
+		colx = tab->file->fields[fieldx]->sheet_column;
 		gtk_sheet_set_column_width (sheet, colx,
-			tab->file->fields[fieldx].width * 8);
+			tab->file->fields[fieldx]->width * 8);
 		gtk_sheet_column_button_add_label (sheet, colx,
-			tab->file->fields[fieldx].name);
+			tab->file->fields[fieldx]->name);
 		gtk_sheet_column_set_justification (sheet, colx,
-			tab->file->fields[fieldx].justification);
-	}
+			tab->file->fields[fieldx]->justification);
+	}*/
 	gtk_sheet_show_column_titles (sheet);
 
 	/* create a new scrolled window. */
@@ -768,8 +796,8 @@ build_basic_list_mode (QlTabData * tab)
 
 	gtk_container_add (GTK_CONTAINER (scrolled_window),
 		GTK_WIDGET (sheet));
-	gtk_notebook_append_page(tab->qlc->notebook, 
-		GTK_WIDGET(scrolled_window), 
+	gtk_notebook_append_page(tab->qlc->notebook,
+		GTK_WIDGET(scrolled_window),
 		tab_label_box (tab, tab->file->file_name));
 	gtk_widget_show (scrolled_window);
 	tbindex = gtk_notebook_get_current_page (tab->qlc->notebook);
@@ -838,7 +866,7 @@ open_recent (GtkRecentChooser * c, gpointer data)
 	if (!filename)
 		return;
 	vfsinfo = gnome_vfs_file_info_new ();
-	vfsresult = gnome_vfs_get_file_info (filename, vfsinfo, 
+	vfsresult = gnome_vfs_get_file_info (filename, vfsinfo,
 		GNOME_VFS_FILE_INFO_DEFAULT);
 	if ((vfsinfo->type == GNOME_VFS_FILE_TYPE_REGULAR) || (vfsinfo->size > 0))
 		actually_open (filename, MODE_OPEN, QL_OLD_QLF, qlc);
