@@ -6,6 +6,7 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file fileout.c
+	@brief  Export routines.
 	@author Copyright 2006-2007  Neil Williams <linux@codehelp.co.uk>
 	@author Copyright 1999  Robert Lissner 
 */
@@ -144,8 +145,28 @@ remove_ql_chars (gchar * text)
 	linebufp = g_strstrip (linebufp); /* removes trailing blanks */
 }								/* end of remove_ql_chars */
 
+static void
+print_field_cb (gpointer G_GNUC_UNUSED field_index, gpointer field_ptr, gpointer user_data)
+{
+	QlFieldInfo * field = (QlFieldInfo*)field_ptr;
+	FILE * f; /**< \bug stop passing FILE pointers around. */
+	f = (FILE*)user_data;
+
+	remove_ql_chars (field->name);
+	fprintf (f, "FIELD %u %u %u %u %u %u %u %u %s\n",
+		field->type,
+		field->formatting,
+		field->decimal_places,
+		field->justification,
+		field->sheet_column,
+		field->width,
+		field->unused1,
+		field->unused2, 
+		field->name);
+}
+
 /** Write the file onto disk.  Called by save, save_as, close, quit */
-/** \todo stop passing f, just pass the filename, if at all. */
+/** \bug stop passing f, just pass the filename, if at all. */
 static void
 write_ql_file (QlTabData * tab, FILE *f)
 {
@@ -173,21 +194,23 @@ write_ql_file (QlTabData * tab, FILE *f)
 		tab->file->filter_ct,
 		tab->file->report_ct, tab->view->width, tab->view->height, 
 		tab->view->x, tab->view->y);
+	ql_fieldinfo_foreach (tab, print_field_cb, f);
+/*
 	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
 	{
-		remove_ql_chars (tab->file->fields[fieldx].name);
+		remove_ql_chars (tab->file->fields[fieldx]->name);
 		fprintf (f, "FIELD %u %u %u %u %u %u %u %u %s\n",
-			tab->file->fields[fieldx].type,
-			tab->file->fields[fieldx].formatting,
-			tab->file->fields[fieldx].decimal_places,
-			tab->file->fields[fieldx].justification,
-			tab->file->fields[fieldx].sheet_column,
-			tab->file->fields[fieldx].width,
-			tab->file->fields[fieldx].unused1,
-			tab->file->fields[fieldx].unused2, 
-			tab->file->fields[fieldx].name);
+			tab->file->fields[fieldx]->type,
+			tab->file->fields[fieldx]->formatting,
+			tab->file->fields[fieldx]->decimal_places,
+			tab->file->fields[fieldx]->justification,
+			tab->file->fields[fieldx]->sheet_column,
+			tab->file->fields[fieldx]->width,
+			tab->file->fields[fieldx]->unused1,
+			tab->file->fields[fieldx]->unused2, 
+			tab->file->fields[fieldx]->name);
 	}
-
+*/
 	for (sortx = 0; sortx < tab->file->sort_ct; sortx++)
 	{
 		remove_ql_chars (tab->file->sorts[sortx].name);
@@ -248,11 +271,15 @@ write_ql_file (QlTabData * tab, FILE *f)
 	   The logic is difficult enough that I use an intermediate variable */
 	for (rowx = 0; rowx < tab->file->last_row; rowx++)
 	{
+		gint last_field;
 		got_data = FALSE;		/* to not write empty records */
 		blank_fields = 0;		/* to not write trailing \\\\\ */
-		for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
+		last_field = ql_get_last_field (tab);
+		for (fieldx = 0; fieldx <= last_field; fieldx++)
 		{
-			colx = tab->file->fields[fieldx].sheet_column;
+			QlFieldInfo * field;
+			field = ql_get_fieldinfo (tab, fieldx);
+			colx = field->sheet_column;
 			text = gtk_sheet_cell_get_text (tab->view->sheet, rowx, colx);
 			if (text)
 			{
@@ -287,7 +314,7 @@ write_html (QlTabData * tab, FILE *f)
 	gint32 rowx;
 	gint16 fieldx;
 	gint16 justification;
-
+	QlFieldInfo * field;
 	gint16 colx = 0;			/* indexes into array */
 
 	/* Output Head Info Stuff */
@@ -303,7 +330,8 @@ write_html (QlTabData * tab, FILE *f)
 	for (colx = write_range.col0; colx <= write_range.coli; colx++)
 	{
 		fieldx = tab->file->col_to_field[colx];
-		fprintf (f, "    <th>%s</th>\n", tab->file->fields[fieldx].name);
+		field = ql_get_fieldinfo (tab, fieldx);
+		fprintf (f, "    <th>%s</th>\n", field->name);
 	}
 	fprintf (f, "  </tr>\n");
 
@@ -319,7 +347,8 @@ write_html (QlTabData * tab, FILE *f)
 			if (text)
 			{
 				fieldx = tab->file->col_to_field[colx];
-				justification = tab->file->fields[fieldx].justification;
+				field = ql_get_fieldinfo (tab, fieldx);
+				justification = field->justification;
 				if (justification == GTK_JUSTIFY_CENTER)
 					fputs ("<center>", f);
 				else if (justification == GTK_JUSTIFY_RIGHT)
