@@ -90,18 +90,25 @@ filter_compare_record (QlTabData * tab, gint32 field_index,
 	gint32 record_index, gchar * criterionx, QLFilterRule rule, 
 	gboolean use_nocase)
 {
-	gint32 column = tab->file->fields[field_index].sheet_column;
-	gchar *entryx = g_strdup (gtk_sheet_cell_get_text 
+	gint32 column;
+	gchar *entry;
+	gchar *entryx; /**< \todo why the duplicate ? */
+	gchar *criterion;
+	QlFieldInfo * field;
+
+	field = ql_get_fieldinfo (tab, field_index);
+	column = field->sheet_column;
+	entryx = g_strdup (gtk_sheet_cell_get_text 
 		(tab->view->sheet, record_index, column));
-	gchar *entry = entryx;
-	gchar *criterion = criterionx;
+	entry = entryx;
+	criterion = criterionx;
 
 	/* First handle the case of blank or nonblank criteria */
 	if (!entry && rule != FILTER_ISBLANK)
 		return FALSE;
 	if (rule == FILTER_ISBLANK && (!entry || strlen (entry) == 0))
 		return TRUE;
-	switch (tab->file->fields[field_index].type)
+	switch (field->type)
 	{
 		/* We are required to handle string/text types ourselves */
 		case FIELD_TYPE_TEXT:
@@ -165,11 +172,9 @@ filter_compare_record (QlTabData * tab, gint32 field_index,
 		case FIELD_TYPE_TIME:
 		{
 			gdouble d_entry =
-				qls2d (entry, tab->file->fields[field_index].type,
-				tab->file->fields[field_index].formatting);
+				qls2d (entry, field->type, field->formatting);
 			gdouble d_criterion = qls2d (criterion,
-				tab->file->fields[field_index].type,
-				tab->file->fields[field_index].formatting);
+				field->type, field->formatting);
 			switch (rule)
 			{
 			case FILTER_CONTAINS:
@@ -711,8 +716,12 @@ filter_do_edit (QlTabData * tab, gint16 filter_index)
 				field_selector_menu_item = gtk_menu_item_new_with_label
 					(_("--- Unused ---"));
 			else
+			{
+				QlFieldInfo * field;
+				field = ql_get_fieldinfo (tab, (field_index - 1));
 				field_selector_menu_item = gtk_menu_item_new_with_label
-					(tab->file->fields[field_index - 1].name);
+					(field->name);
+			}
 
 			gtk_widget_set_name (field_selector_menu_item, "field");
 			{
