@@ -6,8 +6,9 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file main.c
+	@brief Utility and conversion functions.
 	@author Copyright 2006-2007  Neil Williams <linux@codehelp.co.uk>
-	@author Copyright 1999  Robert Lissner 
+	@author Copyright 1999  Robert Lissner
 */
 /*
     This package is free software; you can redistribute it and/or modify
@@ -37,7 +38,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gtkextra/gtksheet.h>
-#include <monetary.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include "types.h"
 #include "dialog_initial.h"
@@ -152,47 +152,9 @@ display_time (GTimeVal * gt, const gchar * fmt)
 	return g_strdup (buf);
 }
 
-/** \brief used for currency
-
- \todo rename function
-*/
-gchar *
-display_number_str (QlCurrFormat qc, gdouble number)
-{
-	gchar buf[MAX_DATE_BUFFER];
-	gsize len;
-	gchar * format;
-
-	format = NULL;
-	switch (qc)
-	{
-		/** \todo read the decimal places parameter */
-		/* "1,234.56" %! formatting style for formatting == 0 */
-		case QL_NF_GRP :   { format = g_strdup("%!i"); break;  }
-		/* "1234.56"  %^! */
-		case QL_NF :       { format = g_strdup("%^!i"); break; }
-		/* "$1,234.56"  default (%n) */
-		case QL_NF_CURR :  { format = g_strdup("%n"); break;  }
-		/* "1,234.56%" percentage numbers */
-		case QL_NF_PERCENT : { format = g_strdup ("%!i%%"); break; }
-		/* "1.234,56" used for files only */
-		/* "1234,56" used for files only */
-		/* "1.234,56%" used for files only */
-		default : break;
-	}
-	if (!format)
-		return NULL;
-	buf[0] = '\1';
-	len = strfmon (buf, MAX_DATE_BUFFER, format, number);
-	g_free (format);
-	if (len <= 0 && buf[0] != '\0')
-		return NULL;
-	return g_strdup (buf);
-}
-
 /** \brief general purpose date parser
 
-\since 0.9.0
+ \since 0.9.0
 
 Due to be extended further with QofDate from the
 Query Object Framework but currently using GDate.
@@ -208,7 +170,7 @@ parse_date (const gchar * string, const gchar * format)
 	g_date_set_parse (gd, string);
 	if (g_date_valid (gd))
 		return gd;
-	else 
+	else
 	{
 		/** \bug improve this handling with QofDate */
 		const gchar *cp;
@@ -290,11 +252,11 @@ big_draw_start (QlTabData * tab)
 {
 	if (tab->view->display_mode == DISPLAY_LIST)
 	{
-		g_signal_handler_block 
+		g_signal_handler_block
 			(tab->view->sheet, tab->view->activate_signal);
-		g_signal_handler_block 
+		g_signal_handler_block
 			(tab->view->sheet, tab->view->new_column_width_signal);
-		g_signal_handler_block 
+		g_signal_handler_block
 			(tab->view->sheet, tab->view->select_range_signal);
 		gtk_sheet_freeze (tab->view->sheet);
 	}
@@ -308,11 +270,11 @@ big_draw_end (QlTabData * tab)
 	if (tab->view->display_mode == DISPLAY_LIST)
 	{
 		gtk_sheet_thaw (tab->view->sheet);
-		g_signal_handler_unblock 
+		g_signal_handler_unblock
 			(tab->view->sheet, tab->view->activate_signal);
-		g_signal_handler_unblock 
+		g_signal_handler_unblock
 			(tab->view->sheet, tab->view->new_column_width_signal);
-		g_signal_handler_unblock 
+		g_signal_handler_unblock
 			(tab->view->sheet, tab->view->select_range_signal);
 	}
 	else
@@ -344,8 +306,9 @@ check_if_changed (QlTabData * tab)
 {
 	gint fieldx;
 	gdouble temp_double;
-	gchar linebuf[40];
+	gchar linebuf[40]; /**< \bug remove the static buffer */
 	gchar *newtext;
+	QlFieldInfo * field;
 
 	g_return_val_if_fail (tab, FALSE);
 	/* initial vals */
@@ -374,19 +337,20 @@ check_if_changed (QlTabData * tab)
 
 	/* check that input is valid in non-text fields */
 	fieldx = tab->file->col_to_field[tab->view->activate_col];
-	if (tab->file->fields[fieldx].type != FIELD_TYPE_TEXT)
+	field = ql_get_fieldinfo (tab, fieldx);
+	if (field->type != FIELD_TYPE_TEXT)
 	{
-		temp_double = qls2d (newtext, tab->file->fields[fieldx].type,
-			tab->file->fields[fieldx].formatting);
+		temp_double = qls2d (newtext, field->type,
+			field->formatting);
 
 		if (temp_double >= HUGE_VAL)
 		{
 			level2_error (tab, _("The cell contents are not valid."));
 			return (TRUE);
 		}
-		d2qls (linebuf, temp_double, tab->file->fields[fieldx].type,
-			tab->file->fields[fieldx].formatting,
-			tab->file->fields[fieldx].decimal_places);
+		d2qls (linebuf, temp_double, field->type,
+			field->formatting,
+			field->decimal_places);
 		gtk_sheet_set_cell_text (tab->view->sheet,
 			tab->view->activate_row, tab->view->activate_col, linebuf);
 	}
@@ -396,21 +360,20 @@ check_if_changed (QlTabData * tab)
 /** \brief old compatibility function
 
 Converts an internal double to an old quicklist string.
+
+ \bug formatting is overridden for use by QlCurrFormat as well.
+
 */
 gint
-d2qls (gchar * texto, gdouble input, QlFieldType type, 
-	   QlDateFormat formatting,	gint dec_places)
+d2qls (gchar * texto, gdouble input, QlFieldType type,
+	   QlDateFormat formatting, gint dec_places)
 {
 	/* these are for numeric conversions */
 	gboolean is_negative;
-	gchar use_for_currency;
+	gboolean is_currency;
 	gboolean is_percent;
-	gchar use_for_comma;
-	gchar use_for_decimal;
-	gchar texti[40];
-	gint inx, outx;
+	gchar * texti;
 	gdouble temp_val;
-	gint digits_so_far;
 
 	/* these are for date conversion */
 	GDateYear year;
@@ -424,7 +387,7 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 	temp_val = input;
 	if (temp_val >= HUGE_VAL)
 	{
-		strcpy (texto, "######");
+		texto = g_strdup ("######");
 		return 6;
 	}
 	switch (type)
@@ -432,25 +395,21 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 		case FIELD_TYPE_NUMERIC:
 		{
 			is_negative = FALSE;
-			is_percent = FALSE;
-			use_for_currency = '\0';
-			use_for_comma = '\0';
-			use_for_decimal = '.';
-			digits_so_far = 0;
-
 			if (temp_val < 0)
 				is_negative = TRUE;
+			is_percent = FALSE;
+			is_currency = FALSE;
 			switch (formatting)
 			{
 			case 0:
 				{
-					use_for_comma = ',';
+//					use_for_comma = ',';
 					break;
 				}
 			case 1:
 				{
-					use_for_comma = '.';
-					use_for_decimal = ',';
+//					use_for_comma = '.';
+//					use_for_decimal = ',';
 					break;
 				}
 				/** \todo check why this is empty. */
@@ -458,26 +417,26 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 				break;
 			case 3:
 				{
-					use_for_decimal = ',';
+//					use_for_decimal = ',';
 					break;
 				}
 			case 4:
 				{
-					use_for_currency = '$';
-					use_for_comma = ',';
+//					use_for_currency = '$';
+//					use_for_comma = ',';
 					break;
 				}
 			case 5:
 				{
-					use_for_comma = ',';
-					is_percent = TRUE;
+//					use_for_comma = ',';
+//					is_percent = TRUE;
 					break;
 				}
 				/* this would be case 6 */
 			default:
 				{
-					use_for_comma = '.';
-					use_for_decimal = ',';
+//					use_for_comma = '.';
+//					use_for_decimal = ',';
 					is_percent = TRUE;
 				}
 			}
@@ -488,59 +447,59 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 		{
 			case 0:
 			{
-				sprintf (texti, "%.0f", temp_val);
+				texti = g_strdup_printf ("%.0f", temp_val);
 				break;
 			}
 			case 1:
 			{
-				sprintf (texti, "%.1f", temp_val);
+				texti = g_strdup_printf ("%.1f", temp_val);
 				break;
 			}
 			case 2:
 			{
-				sprintf (texti, "%.2f", temp_val);
+				texti = g_strdup_printf ("%.2f", temp_val);
 				break;
 			}
 			case 3:
 			{
-				sprintf (texti, "%.3f", temp_val);
+				texti = g_strdup_printf ("%.3f", temp_val);
 				break;
 			}
 			case 4:
 			{
-				sprintf (texti, "%.4f", temp_val);
+				texti = g_strdup_printf ("%.4f", temp_val);
 				break;
 			}
 			case 5:
 			{
-				sprintf (texti, "%.5f", temp_val);
+				texti = g_strdup_printf ("%.5f", temp_val);
 				break;
 			}
 			case 6:
 			{
-				sprintf (texti, "%.6f", temp_val);
+				texti = g_strdup_printf ("%.6f", temp_val);
 				break;
 			}
 			case 7:
 			{
-				sprintf (texti, "%.7f", temp_val);
+				texti = g_strdup_printf ("%.7f", temp_val);
 				break;
 			}
 			case 8:
 			{
-				sprintf (texti, "%.8f", temp_val);
+				texti = g_strdup_printf ("%.8f", temp_val);
 				break;
 			}
 			/* this would be case 9 */
 			default:
 			{
-				sprintf (texti, "%.9f", temp_val);
+				texti = g_strdup_printf ("%.9f", temp_val);
 				break;
 			}
 		}
 
 		/* now count the digits before the decimal place or the end */
-		if (use_for_comma)
+/*		if (use_for_comma)
 			for (inx = 0; inx < 40; inx++)
 			{
 				if (texti[inx] >= '0' && texti[inx] <= '9')
@@ -548,16 +507,16 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 				else if (!texti[inx] || texti[inx] == '.')
 					break;
 			}
-
+*/
 		/* have counted places in front of decimal.  Now set
 		   up counter of how many digits to skip before inserting
 		   a comma */
-		digits_so_far %= 3;
+/*		digits_so_far %= 3;
 		if (!digits_so_far)
 			digits_so_far = 3;
-
+*/
 		/* start formatting output */
-		outx = 0;
+/*		outx = 0;
 		if (use_for_currency)
 		{
 			if (is_negative)
@@ -580,21 +539,22 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 			}
 			else if (texti[inx] == '.')
 			{
-				/* turn off commas past decimal point */
-				use_for_comma = '\0';
+*/				/* turn off commas past decimal point */
+/*				use_for_comma = '\0';
 				texto[outx++] = use_for_decimal;
 			}
 			else if (!texti[inx])
 				break;
 		}
-
+*/
 		/* finished moving everything.  Put percent or minus on end */
-		if (use_for_currency && is_negative)
+/*		if (use_for_currency && is_negative)
 			texto[outx++] = ')';
 		if (is_percent)
 			texto[outx++] = '%';
-		texto[outx] = '\0';
-		return (outx);
+		texto[outx] = '\0';*/
+		texto = texti;
+		return 0;
 		break;					/* end of FIELD_TYPE_NUMERIC */
 
 		case FIELD_TYPE_DATE:
@@ -602,7 +562,6 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 			GDate * gd;
 			const gchar * fmt;
 			fmt = convert_old_df (formatting);
-			fprintf (stderr, "fmt=%s\n", fmt);
 			year = (gint) temp_val / 10000;
 			month = (gint) temp_val / 100 % 100;
 			day = (gint) temp_val % 100;
@@ -649,13 +608,13 @@ d2qls (gchar * texto, gdouble input, QlFieldType type,
 void
 connect_signals (QlTabData * tab)
 {
-	tab->view->activate_signal = g_signal_connect 
+	tab->view->activate_signal = g_signal_connect
 		(G_OBJECT (tab->view->sheet),
 		"activate", G_CALLBACK (activate_callback), tab);
-	tab->view->new_column_width_signal = g_signal_connect 
+	tab->view->new_column_width_signal = g_signal_connect
 		(G_OBJECT (tab->view->sheet),
 		"new_column_width", G_CALLBACK (new_column_width), tab);
-	tab->view->select_range_signal = g_signal_connect 
+	tab->view->select_range_signal = g_signal_connect
 		(G_OBJECT (tab->view->sheet),
 		"select_range", G_CALLBACK (select_range_callback), tab);
 }								/* end of connect_signals */
@@ -699,18 +658,18 @@ level1_error (QlTabData * tab, gchar * message)
 {
 	GtkWidget * dlg;
 
-	dlg = gtk_message_dialog_new (GTK_WINDOW(tab->qlc->parent), 
+	dlg = gtk_message_dialog_new (GTK_WINDOW(tab->qlc->parent),
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
 		"%s", message);
-	g_signal_connect (GTK_OBJECT (dlg), "response", 
+	g_signal_connect (GTK_OBJECT (dlg), "response",
 		G_CALLBACK (gtk_widget_destroy), dlg);
 	gtk_dialog_run (GTK_DIALOG (dlg));
 	gtk_widget_destroy (dlg);
 }								/* End of level1_error */
 
 /** Error dialog box when there's already a dialogue on
-   the screen, and it has to stay 
+   the screen, and it has to stay
 
 \bug pointless function.
 */
@@ -719,7 +678,7 @@ level2_error (QlTabData * tab, gchar * message)
 {
 	GtkWidget * dlg;
 
-	dlg = gtk_message_dialog_new (GTK_WINDOW(tab->qlc->parent), 
+	dlg = gtk_message_dialog_new (GTK_WINDOW(tab->qlc->parent),
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
 		"%s", message);
@@ -740,7 +699,7 @@ make_basic_dialog1 (void)
 }
 
 /**  converts a quicklist string (1,234.56) to double.
-	\bug Have to check for NULL and newlines at end of 
+	\bug Have to check for NULL and newlines at end of
 	line because of reading input files.
 
 	\todo replace with QOF/GLib functions.
@@ -782,20 +741,20 @@ qls2d (const gchar * input, gint type, gint formatting)
 		GDateMonth month = 3;
 		GDateYear year = 2004;
 		gdouble retval;
-//		fprintf (stderr, "str=%s %d %d\n", input, type, formatting);
+
 		/** \bug default format is day/month/year
-		
+
 		parse_date is locale sensitive, default was not. */
 		gd = parse_date (input, convert_old_df (formatting));
 		if (!gd)
 		{
 			fprintf (stderr, "null gdate\n");
-			fprintf (stderr, "format=%s\n", 
+			fprintf (stderr, "format=%s\n",
 				convert_old_df (formatting));
 			retval = year * 10000 + month * 100 + day;
 			fprintf (stderr, "val=%f\n", retval);
 			return retval;
-		}			
+		}
 		if (g_date_valid (gd))
 		{
 			day = g_date_get_day (gd);
@@ -803,8 +762,7 @@ qls2d (const gchar * input, gint type, gint formatting)
 			year = g_date_get_year (gd);
 		}
 		retval = year * 10000 + month * 100 + day;
-//		fprintf (stderr, "val=%f\n", retval);
-		return retval;
+	return retval;
 	}
 
 	else
@@ -873,9 +831,13 @@ reset_col_to_field (QlTabData * tab)
 {
 	gint fieldx;
 	gint colx;
+	QlFieldInfo * field;
 	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
 	{
-		colx = tab->file->fields[fieldx].sheet_column;
+		field = ql_get_fieldinfo (tab, fieldx);
+		if (!field)
+			continue;
+		colx = field->sheet_column;
 		tab->file->col_to_field[colx] = fieldx;
 	}
 }
