@@ -6,6 +6,7 @@
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /** @file report.c
+	@brief Report support (GtkHTML) - incomplete.
 	@author Copyright 2006, 2007  Neil Williams <linux@codehelp.co.uk>
     @author Copyright 1999  Robert Lissner, Jay MacDonald,
 	@author Copyright 1999  Sam Phillips, Keith Wesolowski. 
@@ -48,12 +49,14 @@
 #include "filter.h"
 #include "main.h"
 #include "menus.h"
+#include "field.h"
 #include "sort.h"
 #include "fileout.h"
 
 static GtkWidget * select_dlg = NULL;
 /* prototype GtkHTML output */
-#define WRITETAG(x)	gtk_html_write (html, handle, (x), strlen((x) + 1));
+#define WRITETAG(x)	gtk_html_write (html, handle, (x), strlen((x)));
+
 /** \todo migrate into the tab context 
 @{
 */
@@ -111,7 +114,7 @@ typedef struct
 	QlTimeFormat time_fmt;
 	/** \todo standardise and localise the format specifier for plain text. */
 	gint txt_format;
-	QlFieldInfo  field_info;
+	QlFieldInfo * field_info;
 	/** \todo remove this placeholder when the stream is part of the tab context. */
 	GtkHTMLStream * temp;
 }QlTHead;
@@ -190,22 +193,21 @@ tab_label_box (QlTabData * tab, const gchar * label_text)
 static void
 thead_cb (gpointer theadp, gpointer tabp)
 {
-	gint length;
 	gchar * buf;
 	QlTabData * tab = (QlTabData*) tabp;
+	QlFieldInfo * field;
 	QlTHead * thead = (QlTHead*)theadp;
 	GtkHTMLStream * handle = thead->temp;
 
-	buf = g_strconcat ("<td><b>", 
-		tab->file->fields[thead->field_info.sheet_column].name, "</b></td>\n", NULL);
-	length = strlen(buf);
-	gtk_html_write (html, handle, buf, length);
+	field = ql_get_fieldinfo (tab, thead->field_info->sheet_column);
+	buf = g_strconcat ("<td><b>", field->name, "</b></td>\n", NULL);
+	WRITETAG(buf);
+	g_free (buf);
 }
 
 static void
 tbl_row_cb (gpointer theadp, gpointer tabp)
 {
-	gint length;
 	gchar * buf, * tag;
 	QlTabData * tab = (QlTabData*) tabp;
 	QlTHead * thead = (QlTHead*)theadp;
@@ -213,23 +215,28 @@ tbl_row_cb (gpointer theadp, gpointer tabp)
 
 	buf = NULL;
 	/* <table> needs data in rows */
+	/** \bug buf is always empty */
 	buf = gtk_sheet_cell_get_text (tab->view->sheet, row,
-		thead->field_info.sheet_column);
+		thead->field_info->sheet_column);
 	if (!buf)
 		buf = "&nbsp;";
 	tag = g_strconcat ("<td>", buf, "</td>\n", NULL);
-	length = strlen(tag);
-	gtk_html_write (html, handle, tag, length);
+	WRITETAG(tag);
+	g_free (buf);
 }
 
-/* Add a new HTML table */
+/** \brief Add a new HTML table 
+
+ \param tab - the noteboook tab of the file view.
+
+Creates a new tab for the report view.
+*/
 static void
 html_on_screen (QlTabData * tab)
 {
 	GtkWidget *scrolled_window;
 	GtkWidget *html_widget;
 	gchar * buf;
-	gsize taglength;
 	gshort c, field_num;
 	GtkHTMLStream * handle = NULL;
 
@@ -242,49 +249,49 @@ html_on_screen (QlTabData * tab)
 	gtk_html_load_empty (html);
 	gtk_html_set_editable (html, FALSE);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), html_widget);
-	handle = gtk_html_begin_content (html, "text/html; charset=utf-8");
+	handle = gtk_html_begin_content (html, "text/html; charset=UTF-8");
 	WRITETAG("<html><head>\n");
 	WRITETAG("<meta name=\"Generator\" content=\"QUICKLIST\">\n");
-	/** \todo report_ptr will be a GList soon. */
-	buf = g_strconcat ("<title>", tab->file->report_ptr->header, "</title>\n", NULL);
-	taglength = strlen (buf) + 1;
-	gtk_html_write (html, handle, buf, taglength);
 	WRITETAG("</head><body>\n");
 	buf = g_strconcat ("<h1>", tab->file->report_ptr->header, "</h1>\n", NULL);
-	taglength = strlen (buf) + 1;
-	gtk_html_write (html, handle, buf, taglength);
+	WRITETAG(buf);
+	g_free (buf);
 	buf = g_strconcat ("<h2>", tab->file->report_ptr->name, "</h2>\n", NULL);
-	taglength = strlen (buf) + 1;
-	gtk_html_write (html, handle, buf, taglength);
+	WRITETAG(buf);
+	g_free (buf);
 	for (c = 0; c < tab->file->report_ptr->last_column; c++)
 	{
+		QlFieldInfo * field;
 		QlTHead * thead = g_new0 (QlTHead, 1);
 		/** \bug QlReportColumn starts at 1, not zero. */
-		thead->field_info.sheet_column = c + 1;
+		field = g_new0(QlFieldInfo, 1);
+		thead->field_info = field;
+		thead->field_info->sheet_column = c + 1;
 		field_num = tab->file->report_ptr->column[c+1].field;
-		thead->field_info.type = tab->file->fields[field_num].type;
+		field = ql_get_fieldinfo (tab, field_num);
+		thead->field_info->type = field->type;
 		thead->temp = handle;
 		/* not used yet - could support alternative formatting in report. */
-		switch (thead->field_info.type)
+		switch (thead->field_info->type)
 		{
 			case FIELD_TYPE_NUMERIC :
 			{
-				thead->curr_fmt = tab->file->fields[field_num].formatting;
+				thead->curr_fmt = field->formatting;
 				break;
 			}
 			case FIELD_TYPE_DATE :
 			{
-				thead->date_fmt = tab->file->fields[field_num].formatting;
+				thead->date_fmt = field->formatting;
 				break;
 			}
 			case FIELD_TYPE_TIME :
 			{
-				thead->time_fmt = tab->file->fields[field_num].formatting;
+				thead->time_fmt = field->formatting;
 				break;
 			}
 			case FIELD_TYPE_TEXT :
 			{
-				thead->txt_format = tab->file->fields[field_num].formatting;
+				thead->txt_format = field->formatting;
 				break;
 			}
 		}
@@ -303,6 +310,7 @@ html_on_screen (QlTabData * tab)
 		WRITETAG ("</tr>\n");
 	}
 	WRITETAG("</table>\n</body></html>\n");
+	/** \bug data appears set but is not being displayed */
 	gtk_notebook_append_page (tab->qlc->notebook, scrolled_window, 
 		tab_label_box(tab, tab->file->report_ptr->name));
 	gtk_widget_show_all (scrolled_window);
@@ -379,7 +387,7 @@ print_preview_cb (GtkWidget G_GNUC_UNUSED * widget, gpointer G_GNUC_UNUSED data)
 	g_object_unref (print_master);
 }
 
-/* old code */
+/** \todo remove old report_set_col_field code */
 
 /* Sets current column and current field. */
 static void
@@ -397,17 +405,19 @@ display_column (QlTabData * tab, gint colx)
 	gint list_mode_col;
 	gchar *text;
 	QlReportColumn *this_column;
+	QlFieldInfo * field;
 
 	this_column = &tab->file->report_ptr->column[colx];
 
 	fieldx = this_column->field;
-	list_mode_col = tab->file->fields[fieldx].sheet_column;
+	field = ql_get_fieldinfo (tab, fieldx);
+	list_mode_col = field->sheet_column;
 	gtk_sheet_set_column_width (tab->view->report_sheet,
 		colx, this_column->width * 8);
 	gtk_sheet_column_button_add_label (tab->view->report_sheet,
-		colx, tab->file->fields[fieldx].name);
+		colx, field->name);
 	gtk_sheet_column_set_justification (tab->view->report_sheet,
-		colx, tab->file->fields[fieldx].justification);
+		colx, field->justification);
 
 	/* now display the six rows that we are showing as examples */
 	if (this_column->group)
@@ -595,8 +605,10 @@ select_report_range (GtkSheet G_GNUC_UNUSED * sheet, GtkSheetRange * range,
 		report_set_col_field (tab, local_range.col0);
 		dim_all_menus (tab->qlc);
 	}
+/** \bug #456095 setting negative value in array is impossible.
+ This cannot be a sane way of unsetting the selection.
 	else
-		report_set_col_field (tab, -1);	/* no selection */
+		report_set_col_field (tab, -1);	*//* no selection */
 }
 
 static void
@@ -734,7 +746,7 @@ adjust_preview (GtkObject G_GNUC_UNUSED * object, gpointer data)
 		g_free (text);
 		return;
 	}
-	strcpy (tab->file->report_ptr->name, text);
+	tab->file->report_ptr->name = g_strdup(text);
 	g_free (text);
 	text = g_strdup (gtk_entry_get_text (GTK_ENTRY (tab->view->entry2)));
 	if (check_entry (tab, text))
@@ -742,7 +754,7 @@ adjust_preview (GtkObject G_GNUC_UNUSED * object, gpointer data)
 		g_free (text);
 		return;
 	}
-	strcpy (tab->file->report_ptr->header, text);
+	tab->file->report_ptr->header = g_strdup(text);
 	g_free (text);
 	text = g_strdup (gtk_entry_get_text (GTK_ENTRY (tab->view->entry3)));
 	if (check_entry (tab, text))
@@ -750,7 +762,7 @@ adjust_preview (GtkObject G_GNUC_UNUSED * object, gpointer data)
 		g_free (text);
 		return;
 	}
-	strcpy (tab->file->report_ptr->footer, text);
+	tab->file->report_ptr->footer = g_strdup(text);
 	g_free (text);
 	html_on_screen (tab);
 }
@@ -1001,7 +1013,7 @@ static void
 report_dlg_cancel (GtkWidget G_GNUC_UNUSED * w, gpointer data)
 {
 	gtk_widget_destroy (GTK_WIDGET(data));
-}	
+}
 
 static void
 report_add_cancel (GtkWidget G_GNUC_UNUSED * w, gpointer data)
@@ -1045,8 +1057,7 @@ report_add (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	width = 0;
 	CHECK_CHANGED(tab);
 	tab->file->report_no = tab->file->report_ct++;
-	tab->file->report_ptr = tab->file->report_ptr =
-		&tab->file->reports[tab->file->report_no];
+	tab->file->report_ptr = &tab->file->reports[tab->file->report_no];
 
 	tab->file->report_ptr = g_new0 (QlReportInfo, 1);
 	tab->file->report_ptr->last_column = -1;
@@ -1123,8 +1134,10 @@ report_add (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	newrow[1] = NULL;
 	for (colx = 0; colx <= tab->file->last_field; colx++)
 	{
+		QlFieldInfo * field;
 		fieldx = tab->file->col_to_field[colx];
-		newrow[0] = tab->file->fields[fieldx].name;
+		field = ql_get_fieldinfo (tab, fieldx);
+		newrow[0] = field->name;
 		gtk_tree_store_append (treestore, &child_iter, &parent_iter);
 		gtk_tree_store_set (treestore, &child_iter, REPORT_COL, 
 			newrow[0], -1);
@@ -1212,8 +1225,11 @@ report_col_add (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	newrow[1] = NULL;
 	for (colx = 0; colx <= tab->file->last_field; colx++)
 	{
+		QlFieldInfo * field;
+		/** \todo use a callback here */
 		fieldx = tab->file->col_to_field[colx];
-		newrow[0] = tab->file->fields[fieldx].name;
+		field = ql_get_fieldinfo (tab, fieldx);
+		newrow[0] = field->name;
 		gtk_tree_store_append (treestore, &child_iter, &parent_iter);
 		gtk_tree_store_set (treestore, &child_iter, FROM_COL, 
 			newrow[0], -1);
@@ -1303,7 +1319,7 @@ report_col_del (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	report_go_to (tab, 0, 0, this_col, this_col);
 }								/* end of report_col_del */
 
-/* User clicked OK to select a specific report format, 
+/** \brief User clicked OK to select a specific report format, 
 go print on screen */
 static void
 select_ok (GtkButton G_GNUC_UNUSED * button, gpointer data)
@@ -1320,9 +1336,8 @@ select_ok (GtkButton G_GNUC_UNUSED * button, gpointer data)
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 	{
 		reportx = -1;
-		gtk_tree_model_get (model, &iter, REPORT_COL, 
-			&report_name, -1);
-		for (reportx = 0; reportx <= tab->file->report_ct; reportx++)
+		gtk_tree_model_get (model, &iter, REPORT_COL, &report_name, -1);
+		for (reportx = 0; reportx < tab->file->report_ct; reportx++)
 		{
 			if (0 == strcmp (tab->file->reports[reportx].name, report_name))
 			{
@@ -1341,6 +1356,8 @@ select_ok (GtkButton G_GNUC_UNUSED * button, gpointer data)
 	}
 }
 
+/** \brief Offer the list of reports for the user to select
+*/
 void
 report_select (GtkAction G_GNUC_UNUSED * w, gpointer data)
 {
@@ -1424,6 +1441,7 @@ report_select (GtkAction G_GNUC_UNUSED * w, gpointer data)
 	g_signal_connect (GTK_OBJECT (select_dlg), "delete_event", 
 		G_CALLBACK(report_dlg_cancel), select_dlg);
 	/* now populate the model with report names */
+	/** \todo Convert tab->file->reports to a GHashTable */
 	newrow[1] = NULL;
 	for (reportx = 0; reportx < tab->file->report_ct; reportx++)
 	{
