@@ -128,12 +128,14 @@ field_menu (QlTabData * tab, gint level)
 
 	/* add the individual fields to the menu */
 	field_box = GTK_COMBO_BOX (gtk_combo_box_new_text ());
-	/** \todo replace with a g_list_foreach */
+	/** \todo replace with a callback */
 	for (fieldx = 0; fieldx <= tab->file->last_field; fieldx++)
 	{
+		QlFieldInfo * field;
 		ret_value = level * 100 + fieldx + 1;
+		field = ql_get_fieldinfo (tab, fieldx);
 		gtk_combo_box_insert_text (field_box, ret_value,
-			tab->file->fields[fieldx].name);
+			field->name);
 	}
 	gtk_widget_show (GTK_WIDGET (field_box));
 	g_signal_connect (GTK_OBJECT (field_box),
@@ -265,10 +267,12 @@ do_the_nasty_sort (QlTabData * tab)
 		return;
 	for (levelx = 0; levelx < levels; levelx++)
 	{
+		QlFieldInfo * field;
 		fieldx = curr_sort->line[levelx].field;
-		curr_sort->line[levelx].column = tab->file->fields[fieldx].sheet_column;
-		level_type[levelx] = tab->file->fields[fieldx].type;
-		level_formatting[levelx] = tab->file->fields[fieldx].formatting;
+		field = ql_get_fieldinfo (tab, fieldx);
+		curr_sort->line[levelx].column = field->sheet_column;
+		level_type[levelx] = field->type;
+		level_formatting[levelx] = field->formatting;
 		if (curr_sort->line[levelx].ascending)
 			level_ascending[levelx] = 1;
 		else
